import zlib
import base64


DATA = "H4sIAAAAAAAAClWNMQvCMBCF/0rI4GT1ckao1QqCS3cnt2CvEmmTmqQiiP/dtHHp9N73+Lg7nOnWKkf1qbODCUxNUXIJsALO6B2cqkxjS/5ZPAcb9mRe1NqeEhUspZC4kdvUl//tMfigG31TQVsz1+dip4y6k5srKCSk9uWsJ6dtXXIElLBDjIv1VRwEZ568jw9GihCso7ECiGgFMjW5iSPojnxQXZ/uZLDLEC8iL3BbiPzK2fr4A/5h8j4OAQAA"


def main():
    print(decode_auditlog(DATA))


def decode_auditlog(data):
    return zlib.decompress(base64.b64decode(data), 16)


main()
