class InvalidRequestException(Exception):
    pass


class DailyGoalsExistsException(Exception):
    pass


class DailyGoalsNonExistsException(Exception):
    pass


class InsertDailyGoalsException(Exception):
    pass


class UpdateDailyGoalsException(Exception):
    pass


class DeleteDailyGoalsException(Exception):
    pass


class NotCacheGoals(Exception):
    pass


class GetSalesChallengeException(Exception):
    pass


class InsertSalesChallengeException(Exception):
    pass


class DeleteSalesChallengeException(Exception):
    pass


class UpdateSalesChallengeException(Exception):
    pass


class GetPaidOrdersException(Exception):
    pass


class GetOrderItemsException(Exception):
    pass
