import pyscripts

from mbcontextmessagehandler import MbContextMessageBus
from nreports_api import NReports


def get_nreports_service():
    # type: (...) -> NReports

    message_bus = MbContextMessageBus(mbcontext=pyscripts.mbcontext)
    return NReports(message_bus=message_bus)
