from typing import (
    Any,
    Dict,
)


class LockBenefitRequest(object):
    def __init__(
        self,           # type: LockBenefitRequest
        benefit,        # type: Dict[str, Any]
        pos_id,         # type: int
        payment_info,   # type: Dict[str, Any]
    ):
        # type: (...) -> None

        self.benefit = benefit
        self.pos_id = pos_id
        self.payment_info = payment_info
