from helper import ExtendedEnum


class ProductTypeId(ExtendedEnum):
    REGULAR = 0
    OPTION = 1
    COMBO = 2
    MENU = 3
    SET = 4
    PROMO = 5
    COUPON = 6


class ProductTypeName:
    OPTION = "OPTION"
    COMBO = "COMBO"
    MENU = "MENU"
    SET = "SET"
    PROMO = "PROMO"
    COUPON = "COUPON"
    PRODUCT = "PRODUCT"
    CANADD = "CANADD"
    INGREDIENT = "INGREDIENT"
