from helper import ExtendedEnum
from messagebus import TokenCreator, TokenPriority

MESSAGE_GROUP = "95"


class ListenedTokens(ExtendedEnum):
    TK_POS_UPDATER_PERFORM_CATALOG_UPDATE = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "1")
    TK_POS_UPDATER_PERFORM_MEDIA_UPDATE = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "2")
    TK_POS_UPDATER_PERFORM_USER_UPDATE = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "3")
    TK_POS_GET_CATALOG_VERSION_TO_APPLY = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "4")
    TK_POS_UPDATER_PERFORM_LOADER_UPDATE = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "5")
    TK_POS_UPDATER_GET_VERSIONS = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "6")
    TK_POS_UPDATER_PERFORM_BINARY_UPDATE = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "7")
    TK_POS_HAS_BINARY_UPDATE_TO_APPLY = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "8")
    TK_POS_UPDATER_PERFORM_CHROME_UPDATE = TokenCreator.create_token(TokenPriority.low, MESSAGE_GROUP, "9")
    TK_POS_UPDATER_HAS_PENDING_LOADER_UPDATE = TokenCreator.create_token(
        priority=TokenPriority.low,
        message_group=MESSAGE_GROUP,
        number="10",
    )
