data = (
'tyal',    # 0x00
'tyalg',    # 0x01
'tyalm',    # 0x02
'tyalb',    # 0x03
'tyals',    # 0x04
'tyalt',    # 0x05
'tyalp',    # 0x06
'tyalh',    # 0x07
'tyam',    # 0x08
'tyab',    # 0x09
'tyabs',    # 0x0a
'tyas',    # 0x0b
'tyass',    # 0x0c
'tyang',    # 0x0d
'tyaj',    # 0x0e
'tyac',    # 0x0f
'tyak',    # 0x10
'tyat',    # 0x11
'tyap',    # 0x12
'tyah',    # 0x13
'tyae',    # 0x14
'tyaeg',    # 0x15
'tyaegg',    # 0x16
'tyaegs',    # 0x17
'tyaen',    # 0x18
'tyaenj',    # 0x19
'tyaenh',    # 0x1a
'tyaed',    # 0x1b
'tyael',    # 0x1c
'tyaelg',    # 0x1d
'tyaelm',    # 0x1e
'tyaelb',    # 0x1f
'tyaels',    # 0x20
'tyaelt',    # 0x21
'tyaelp',    # 0x22
'tyaelh',    # 0x23
'tyaem',    # 0x24
'tyaeb',    # 0x25
'tyaebs',    # 0x26
'tyaes',    # 0x27
'tyaess',    # 0x28
'tyaeng',    # 0x29
'tyaej',    # 0x2a
'tyaec',    # 0x2b
'tyaek',    # 0x2c
'tyaet',    # 0x2d
'tyaep',    # 0x2e
'tyaeh',    # 0x2f
'teo',    # 0x30
'teog',    # 0x31
'teogg',    # 0x32
'teogs',    # 0x33
'teon',    # 0x34
'teonj',    # 0x35
'teonh',    # 0x36
'teod',    # 0x37
'teol',    # 0x38
'teolg',    # 0x39
'teolm',    # 0x3a
'teolb',    # 0x3b
'teols',    # 0x3c
'teolt',    # 0x3d
'teolp',    # 0x3e
'teolh',    # 0x3f
'teom',    # 0x40
'teob',    # 0x41
'teobs',    # 0x42
'teos',    # 0x43
'teoss',    # 0x44
'teong',    # 0x45
'teoj',    # 0x46
'teoc',    # 0x47
'teok',    # 0x48
'teot',    # 0x49
'teop',    # 0x4a
'teoh',    # 0x4b
'te',    # 0x4c
'teg',    # 0x4d
'tegg',    # 0x4e
'tegs',    # 0x4f
'ten',    # 0x50
'tenj',    # 0x51
'tenh',    # 0x52
'ted',    # 0x53
'tel',    # 0x54
'telg',    # 0x55
'telm',    # 0x56
'telb',    # 0x57
'tels',    # 0x58
'telt',    # 0x59
'telp',    # 0x5a
'telh',    # 0x5b
'tem',    # 0x5c
'teb',    # 0x5d
'tebs',    # 0x5e
'tes',    # 0x5f
'tess',    # 0x60
'teng',    # 0x61
'tej',    # 0x62
'tec',    # 0x63
'tek',    # 0x64
'tet',    # 0x65
'tep',    # 0x66
'teh',    # 0x67
'tyeo',    # 0x68
'tyeog',    # 0x69
'tyeogg',    # 0x6a
'tyeogs',    # 0x6b
'tyeon',    # 0x6c
'tyeonj',    # 0x6d
'tyeonh',    # 0x6e
'tyeod',    # 0x6f
'tyeol',    # 0x70
'tyeolg',    # 0x71
'tyeolm',    # 0x72
'tyeolb',    # 0x73
'tyeols',    # 0x74
'tyeolt',    # 0x75
'tyeolp',    # 0x76
'tyeolh',    # 0x77
'tyeom',    # 0x78
'tyeob',    # 0x79
'tyeobs',    # 0x7a
'tyeos',    # 0x7b
'tyeoss',    # 0x7c
'tyeong',    # 0x7d
'tyeoj',    # 0x7e
'tyeoc',    # 0x7f
'tyeok',    # 0x80
'tyeot',    # 0x81
'tyeop',    # 0x82
'tyeoh',    # 0x83
'tye',    # 0x84
'tyeg',    # 0x85
'tyegg',    # 0x86
'tyegs',    # 0x87
'tyen',    # 0x88
'tyenj',    # 0x89
'tyenh',    # 0x8a
'tyed',    # 0x8b
'tyel',    # 0x8c
'tyelg',    # 0x8d
'tyelm',    # 0x8e
'tyelb',    # 0x8f
'tyels',    # 0x90
'tyelt',    # 0x91
'tyelp',    # 0x92
'tyelh',    # 0x93
'tyem',    # 0x94
'tyeb',    # 0x95
'tyebs',    # 0x96
'tyes',    # 0x97
'tyess',    # 0x98
'tyeng',    # 0x99
'tyej',    # 0x9a
'tyec',    # 0x9b
'tyek',    # 0x9c
'tyet',    # 0x9d
'tyep',    # 0x9e
'tyeh',    # 0x9f
'to',    # 0xa0
'tog',    # 0xa1
'togg',    # 0xa2
'togs',    # 0xa3
'ton',    # 0xa4
'tonj',    # 0xa5
'tonh',    # 0xa6
'tod',    # 0xa7
'tol',    # 0xa8
'tolg',    # 0xa9
'tolm',    # 0xaa
'tolb',    # 0xab
'tols',    # 0xac
'tolt',    # 0xad
'tolp',    # 0xae
'tolh',    # 0xaf
'tom',    # 0xb0
'tob',    # 0xb1
'tobs',    # 0xb2
'tos',    # 0xb3
'toss',    # 0xb4
'tong',    # 0xb5
'toj',    # 0xb6
'toc',    # 0xb7
'tok',    # 0xb8
'tot',    # 0xb9
'top',    # 0xba
'toh',    # 0xbb
'twa',    # 0xbc
'twag',    # 0xbd
'twagg',    # 0xbe
'twags',    # 0xbf
'twan',    # 0xc0
'twanj',    # 0xc1
'twanh',    # 0xc2
'twad',    # 0xc3
'twal',    # 0xc4
'twalg',    # 0xc5
'twalm',    # 0xc6
'twalb',    # 0xc7
'twals',    # 0xc8
'twalt',    # 0xc9
'twalp',    # 0xca
'twalh',    # 0xcb
'twam',    # 0xcc
'twab',    # 0xcd
'twabs',    # 0xce
'twas',    # 0xcf
'twass',    # 0xd0
'twang',    # 0xd1
'twaj',    # 0xd2
'twac',    # 0xd3
'twak',    # 0xd4
'twat',    # 0xd5
'twap',    # 0xd6
'twah',    # 0xd7
'twae',    # 0xd8
'twaeg',    # 0xd9
'twaegg',    # 0xda
'twaegs',    # 0xdb
'twaen',    # 0xdc
'twaenj',    # 0xdd
'twaenh',    # 0xde
'twaed',    # 0xdf
'twael',    # 0xe0
'twaelg',    # 0xe1
'twaelm',    # 0xe2
'twaelb',    # 0xe3
'twaels',    # 0xe4
'twaelt',    # 0xe5
'twaelp',    # 0xe6
'twaelh',    # 0xe7
'twaem',    # 0xe8
'twaeb',    # 0xe9
'twaebs',    # 0xea
'twaes',    # 0xeb
'twaess',    # 0xec
'twaeng',    # 0xed
'twaej',    # 0xee
'twaec',    # 0xef
'twaek',    # 0xf0
'twaet',    # 0xf1
'twaep',    # 0xf2
'twaeh',    # 0xf3
'toe',    # 0xf4
'toeg',    # 0xf5
'toegg',    # 0xf6
'toegs',    # 0xf7
'toen',    # 0xf8
'toenj',    # 0xf9
'toenh',    # 0xfa
'toed',    # 0xfb
'toel',    # 0xfc
'toelg',    # 0xfd
'toelm',    # 0xfe
'toelb',    # 0xff
)
