from production.manager import ProductionManager
from production.command import DevicesUpdatedCommand


class DeviceStatusChangedInteractor(object):
    def __init__(
        self,
        production_manager,     # type: ProductionManager
    ):
        # type: (...) -> None

        self.production_manager = production_manager

    def execute(
        self,   # type: DeviceStatusChangedInteractor
    ):
        # type: (...) -> None

        self.production_manager.handle_command(command=DevicesUpdatedCommand())
